@@ -1,3 +1,4 @@
+import logging
 from typing import Any, Dict, List, Optional, Tuple, Union, cast
 
 import requests
@@ -5,6 +6,8 @@ import requests
 from .types import APIConfig, APIVersion, Country, Environment, Facet, Flavor, JSONType
 from .utils import URLBuilder, http_session
 
+logger = logging.getLogger(__name__)
+
 
 def get_http_auth(environment: Environment) -> Optional[Tuple[str, str]]:
     return ("off", "off") if environment is Environment.net else None
@@ -15,6 +18,7 @@ def send_get_request(
     api_config: APIConfig,
     params: Optional[Dict[str, Any]] = None,
     return_none_on_404: bool = False,
+    auth: Optional[Tuple[str, str]] = None,
 ) -> Optional[JSONType]:
     """Send a GET request to the given URL.
 
@@ -30,7 +34,7 @@ def send_get_request(
         params=params,
         headers={"User-Agent": api_config.user_agent},
         timeout=api_config.timeout,
-        auth=get_http_auth(api_config.environment),
+        auth=auth,
     )
     if r.status_code == 404 and return_none_on_404:
         return None
@@ -38,7 +42,7 @@ def send_get_request(
     return r.json()
 
 
-def send_for_urlencoded_post_request(
+def send_form_urlencoded_post_request(
     url: str, body: Dict[str, Any], api_config: APIConfig
 ) -> requests.Response:
     cookies = None
@@ -61,6 +65,26 @@ def send_for_urlencoded_post_request(
     return r
 
 
+class RobotoffResource:
+    def __init__(self, api_config: APIConfig):
+        self.api_config = api_config
+        self.base_url = URLBuilder.robotoff(environment=api_config.environment)
+
+    def predict_lang(self, text: str, k: int = 10, threshold: float = 0.01) -> JSONType:
+        """Predict the language of a text.
+
+        :param text: the text to predict the language of
+        :param k: the number of predictions to return, defaults to 10
+        :param threshold: the minimum probability for a prediction to be
+            returned, defaults to 0.01
+        :return: the API response
+        """
+        return http_session.post(
+            url=f"{self.base_url}/api/v1/predict/lang",
+            data={"text": text, "k": k, "threshold": threshold},
+        ).json()
+
+
 class FacetResource:
     def __init__(self, api_config: APIConfig):
         self.api_config = api_config
@@ -70,13 +94,28 @@ class FacetResource:
             country_code=self.api_config.country.name,
         )
 
-    def get(self, facet_name: Union[Facet, str]) -> JSONType:
+    def get(
+        self,
+        facet_name: Union[Facet, str],
+        page: int = 1,
+        page_size: int = 20,
+        **kwargs: JSONType,
+    ) -> JSONType:
+        """Return all possible values for a given facet.
+
+        :param facet_name: the facet name, e.g. "label"
+        :param page: the page number, defaults to 1
+        :param page_size: the number of items per page, defaults to 20
+        :param kwargs: additional query parameters to pass to the API
+        :return: the API response containing all possible values for the facet
+        """
         facet = Facet.from_str_or_enum(facet_name)
         facet_plural = facet.value.replace("_", "-")
         resp = send_get_request(
-            url=f"{self.base_url}/{facet_plural}",
-            params={"json": "1"},
+            url=f"{self.base_url}/facets/{facet_plural}",
+            params={"json": "1", "page": page, "page_size": page_size, **kwargs},
             api_config=self.api_config,
+            auth=get_http_auth(self.api_config.environment),
         )
         resp = cast(JSONType, resp)
         return resp
@@ -88,6 +127,7 @@ class FacetResource:
         page: int = 1,
         page_size: int = 25,
         fields: Optional[List[str]] = None,
+        sort_by: Optional[str] = None,
     ) -> JSONType:
         """Return products for a given facet value.
 
@@ -97,18 +137,24 @@ class FacetResource:
         :param page_size: the number of items per page, defaults to 25
         :param fields: a list of fields to return. If None, all fields are
             returned.
+        :param sort_by: the sorting key, defaults to None (no sorting)
+            possible values (not exhaustive) are: "popularity",
+            "last_modified_t", "created_t".
         :return: the API response
         """
         facet = Facet.from_str_or_enum(facet_name)
-        facet_singular = facet.name.replace("_", "-")
-        params: JSONType = {"page": page, "page_size": page_size}
+        facet_plural = facet.value.replace("_", "-")
+        params: JSONType = {"page": page, "page_size": page_size, "json": "1"}
         if fields is not None:
             params["fields"] = ",".join(fields)
+        if sort_by is not None:
+            params["sort_by"] = sort_by
 
         resp = send_get_request(
-            url=f"{self.base_url}/{facet_singular}/{facet_value}.json",
+            url=f"{self.base_url}/facets/{facet_plural}/{facet_value}",
             params=params,
             api_config=self.api_config,
+            auth=get_http_auth(self.api_config.environment),
         )
         resp = cast(JSONType, resp)
         return resp
@@ -200,6 +246,7 @@ class ProductResource:
             url=f"{self.base_url}/cgi/search.pl",
             api_config=self.api_config,
             params=params,
+            auth=get_http_auth(self.api_config.environment),
         )
 
     def update(self, body: Dict[str, Any]):
@@ -208,7 +255,7 @@ class ProductResource:
             raise ValueError("missing code from body")
 
         url = f"{self.base_url}/cgi/product_jqm2.pl"
-        return send_for_urlencoded_post_request(url, body, self.api_config)
+        return send_form_urlencoded_post_request(url, body, self.api_config)
 
     def select_image(
         self,
@@ -287,6 +334,88 @@ class ProductResource:
         r.raise_for_status()
         return r
 
+    def parse_ingredients(
+        self, text: str, lang: str, timeout: int = 10
+    ) -> list[JSONType]:
+        """Parse ingredients text using Product Opener API.
+
+        It is only available for `off` flavor (food).
+
+        The result is a list of ingredients, each ingredient is a dict with the
+        following keys:
+
+        - id: the ingredient ID. Having an ID does not means that the
+            ingredient is recognized, you must check if it exists in the
+            taxonomy.
+        - text: the ingredient text (as it appears in the input ingredients
+            list)
+        - percent_min: the minimum percentage of the ingredient in the product
+        - percent_max: the maximum percentage of the ingredient in the product
+        - percent_estimate: the estimated percentage of the ingredient in the
+            product
+        - vegan (bool): optional key indicating if the ingredient is vegan
+        - vegetarian (bool): optional key indicating if the ingredient is
+            vegetarian
+
+        :param server_type: the server type (project) to use
+        :param text: the ingredients text to parse
+        :param lang: the language of the text (used for parsing) as a 2-letter
+            code
+        :param timeout: the request timeout in seconds, defaults to 10s
+        :raises RuntimeError: a RuntimeError is raised if the parsing fails
+        :return: the list of parsed ingredients
+        """
+        if self.api_config.flavor != Flavor.off:
+            raise ValueError("ingredient parsing is only available for food")
+
+        if self.api_config.version != APIVersion.v3:
+            logger.warning(
+                "ingredient parsing is only available in v3 of the API (here: %s), using v3",
+                self.api_config.version,
+            )
+        # by using "test" as code, we don't save any information to database
+        # This endpoint is specifically designed for testing purposes
+        url = f"{self.base_url}/api/v3/product/test"
+
+        if len(text) == 0:
+            raise ValueError("text must be a non-empty string")
+
+        try:
+            r = http_session.patch(
+                url,
+                auth=get_http_auth(self.api_config.environment),
+                json={
+                    "fields": "ingredients",
+                    "lc": lang,
+                    "tags_lc": lang,
+                    "product": {
+                        "lang": lang,
+                        f"ingredients_text_{lang}": text,
+                    },
+                },
+                timeout=timeout,
+            )
+        except (
+            requests.exceptions.ConnectionError,
+            requests.exceptions.SSLError,
+            requests.exceptions.Timeout,
+        ) as e:
+            raise RuntimeError(
+                f"Unable to parse ingredients: error during HTTP request: {e}"
+            )
+
+        if not r.ok:
+            raise RuntimeError(
+                f"Unable to parse ingredients (non-200 status code): {r.status_code}, {r.text}"
+            )
+
+        response_data = r.json()
+
+        if response_data.get("status") != "success":
+            raise RuntimeError(f"Unable to parse ingredients: {response_data}")
+
+        return response_data["product"].get("ingredients", [])
+
 
 class API:
     def __init__(
@@ -339,6 +468,7 @@ class API:
         self.country = country
         self.product = ProductResource(self.api_config)
         self.facet = FacetResource(self.api_config)
+        self.robotoff = RobotoffResource(self.api_config)
 
 
 def parse_ingredients(text: str, lang: str, api_config: APIConfig) -> list[JSONType]:
