@@ -1,16 +1,22 @@
+import base64
 import enum
+import io
 import itertools
 import json
 import math
 import operator
 import re
 from collections import Counter, defaultdict
+from pathlib import Path
 from typing import Callable, Dict, List, Optional, Tuple, Union
 
 import requests
 
 from .types import JSONType
-from .utils import get_logger, http_session
+from .utils import _pillow_available, get_logger, http_session
+
+if _pillow_available:
+    from PIL import Image
 
 # Some classes documentation were adapted from Google documentation on
 # https://cloud.google.com/vision/docs/reference/rpc/google.cloud.vision.v1#google.cloud.vision.v1.Symbol
@@ -37,7 +43,7 @@ class OCRField(enum.Enum):
 
 
 class OCRRegex:
-    __slots__ = ("regex", "field", "processing_func", "priority", "notify")
+    __slots__ = ("regex", "field", "processing_func", "priority")
 
     def __init__(
         self,
@@ -45,13 +51,11 @@ class OCRRegex:
         field: OCRField,
         processing_func: Optional[Callable] = None,
         priority: Optional[int] = None,
-        notify: bool = False,
     ):
         self.regex: re.Pattern = regex
         self.field: OCRField = field
         self.processing_func: Optional[Callable] = processing_func
         self.priority = priority
-        self.notify = notify
 
 
 class ImageOrientation(enum.Enum):
@@ -103,6 +107,7 @@ class OCRResult:
         "logo_annotations",
         "safe_search_annotation",
         "label_annotations",
+        "face_annotations",
     )
 
     def __init__(self, data: JSONType):
@@ -111,6 +116,7 @@ class OCRResult:
         self.logo_annotations: List[LogoAnnotation] = []
         self.label_annotations: List[LabelAnnotation] = []
         self.safe_search_annotation: Optional[SafeSearchAnnotation] = None
+        self.face_annotations: List[FaceAnnotation] = []
 
         for text_annotation_data in data.get("textAnnotations", []):
             text_annotation = OCRTextAnnotation(text_annotation_data)
@@ -139,6 +145,10 @@ class OCRResult:
                 data["safeSearchAnnotation"]
             )
 
+        for face_annotation_data in data.get("faceAnnotations", []):
+            face_annotation = FaceAnnotation(face_annotation_data)
+            self.face_annotations.append(face_annotation)
+
     def get_full_text(self) -> str:
         return (
             self.full_text_annotation.text
@@ -180,6 +190,9 @@ class OCRResult:
     def get_label_annotations(self) -> List["LabelAnnotation"]:
         return self.label_annotations
 
+    def get_face_annotations(self) -> List["FaceAnnotation"]:
+        return self.face_annotations
+
     def get_safe_search_annotation(self):
         return self.safe_search_annotation
 
@@ -1124,6 +1137,31 @@ class LabelAnnotation:
         self.description = data["description"]
 
 
+class FaceAnnotation:
+    __slots__ = (
+        "detection_confidence",
+        "joy_likelihood",
+        "sorrow_likelihood",
+        "anger_likelihood",
+        "surprise_likelihood",
+        "under_exposed_likelihood",
+        "blurred_likelihood",
+        "headwear_likelihood",
+    )
+
+    def __init__(self, data: JSONType):
+        self.detection_confidence = data.get("detectionConfidence", 0.0)
+        self.joy_likelihood = Likelihood[data.get("joyLikelihood", "UNKNOWN")]
+        self.sorrow_likelihood = Likelihood[data.get("sorrowLikelihood", "UNKNOWN")]
+        self.anger_likelihood = Likelihood[data.get("angerLikelihood", "UNKNOWN")]
+        self.surprise_likelihood = Likelihood[data.get("surpriseLikelihood", "UNKNOWN")]
+        self.under_exposed_likelihood = Likelihood[
+            data.get("underExposedLikelihood", "UNKNOWN")
+        ]
+        self.blurred_likelihood = Likelihood[data.get("blurredLikelihood", "UNKNOWN")]
+        self.headwear_likelihood = Likelihood[data.get("headwearLikelihood", "UNKNOWN")]
+
+
 class SafeSearchAnnotation:
     __slots__ = ("adult", "spoof", "medical", "violence", "racy")
 
@@ -1145,6 +1183,16 @@ class SafeSearchAnnotation:
         ]
 
 
+class Likelihood(enum.IntEnum):
+    UNKNOWN = 1
+    VERY_UNLIKELY = 2
+    UNLIKELY = 3
+    POSSIBLE = 4
+    LIKELY = 5
+    VERY_LIKELY = 6
+
+
+# This class should be deleted and replaced by `Likelihood` in the future.
 class SafeSearchAnnotationLikelihood(enum.IntEnum):
     UNKNOWN = 1
     VERY_UNLIKELY = 2
@@ -1152,3 +1200,83 @@ class SafeSearchAnnotationLikelihood(enum.IntEnum):
     POSSIBLE = 4
     LIKELY = 5
     VERY_LIKELY = 6
+
+
+def get_base64_from_image(
+    image: Union["Image.Image", Path, str], session: Optional[requests.Session] = None
+) -> str:
+    """Return the base64 representation of an image.
+
+    :param image: the image to convert to base64. Can be a PIL Image, a Path
+        object, or a URL
+    :param session: the requests Session to use to download the image if
+        `image` is a URL, optional (the default session is used if not set)
+    :raises TypeError: if `image` is not a PIL Image, a Path object, or a URL
+    :return: the base64 representation of the image
+    """
+    if _pillow_available and isinstance(image, Image.Image):
+        image_bytes_fp = io.BytesIO()
+        image.save(image_bytes_fp, format="PNG")
+        image_bytes = image_bytes_fp.getvalue()
+    elif isinstance(image, Path):
+        image_bytes = image.read_bytes()
+    elif isinstance(image, str):
+        if not session:
+            session = http_session
+        image_bytes = session.get(image).content
+    else:
+        raise TypeError(f"Invalid type: {type(image)}")
+
+    return base64.b64encode(image_bytes).decode("utf-8")
+
+
+class OCRFeature(str, enum.Enum):
+    """The OCR features supported by Google Cloud Vision API."""
+
+    TEXT_DETECTION = enum.auto()
+    LOGO_DETECTION = enum.auto()
+    LABEL_DETECTION = enum.auto()
+    SAFE_SEARCH_DETECTION = enum.auto()
+    FACE_DETECTION = enum.auto()
+
+
+def run_ocr_on_image_batch(
+    images: list[Union["Image.Image", Path, str]],
+    api_key: str,
+    features: Optional[list[OCRFeature]] = None,
+    session: Optional[requests.Session] = None,
+) -> requests.Response:
+    """Run Google Cloud Vision OCR on a batch of images.
+
+    :param images: a list of images to run OCR on. Can be PIL Images, Path
+        or URLs
+    :param api_key: the Google Cloud Vision API key to use
+    :param features: the list of OCR features to use, defaults to
+        [OCRFeature.TEXT_DETECTION]
+    :param session: the requests Session to use to download the images if
+        `images` contains URLs, optional (the default session is used if not
+        set)
+    :return: the requests Response object
+    """
+    if session is None:
+        session = http_session
+
+    if features is None:
+        features = [OCRFeature.TEXT_DETECTION]
+    else:
+        features = list(set(features))
+
+    base64_images = [get_base64_from_image(image, session=session) for image in images]
+    r = session.post(
+        f"https://vision.googleapis.com/v1/images:annotate?key={api_key}",
+        json={
+            "requests": [
+                {
+                    "features": [{"type": feature.name} for feature in features],
+                    "image": {"content": base64_image},
+                }
+                for base64_image in base64_images
+            ]
+        },
+    )
+    return r
